@@ -10,6 +10,7 @@ import (
 	"fmt"
 	"io"
 	"net"
+	"net/url"
 	"os"
 	"strconv"
 	"strings"
@@ -24,6 +25,7 @@ import (
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/util/version"
 	"k8s.io/apimachinery/pkg/watch"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/kubernetes/scheme"
@@ -35,8 +37,8 @@ import (
 	"k8s.io/client-go/util/flowcontrol"
 )
 
-// kubeUnit implements the WorkUnit interface.
-type kubeUnit struct {
+// KubeUnit implements the WorkUnit interface.
+type KubeUnit struct {
 	BaseWorkUnitForWorkUnit
 	authMethod          string
 	streamMethod        string
@@ -54,7 +56,7 @@ type kubeUnit struct {
 }
 
 // kubeExtraData is the content of the ExtraData JSON field for a Kubernetes worker.
-type kubeExtraData struct {
+type KubeExtraData struct {
 	Image         string
 	Command       string
 	Params        string
@@ -64,6 +66,112 @@ type kubeExtraData struct {
 	PodName       string
 }
 
+type KubeAPIer interface {
+	NewNotFound(schema.GroupResource, string) *apierrors.StatusError
+	OneTermEqualSelector(string, string) fields.Selector
+	NewForConfig(*rest.Config) (*kubernetes.Clientset, error)
+	GetLogs(*kubernetes.Clientset, string, string, *corev1.PodLogOptions) *rest.Request
+	Get(context.Context, *kubernetes.Clientset, string, string, metav1.GetOptions) (*corev1.Pod, error)
+	Create(context.Context, *kubernetes.Clientset, string, *corev1.Pod, metav1.CreateOptions) (*corev1.Pod, error)
+	List(context.Context, *kubernetes.Clientset, string, metav1.ListOptions) (*corev1.PodList, error)
+	Watch(context.Context, *kubernetes.Clientset, string, metav1.ListOptions) (watch.Interface, error)
+	Delete(context.Context, *kubernetes.Clientset, string, string, metav1.DeleteOptions) error
+	SubResource(*kubernetes.Clientset, string, string) *rest.Request
+	InClusterConfig() (*rest.Config, error)
+	NewDefaultClientConfigLoadingRules() *clientcmd.ClientConfigLoadingRules
+	BuildConfigFromFlags(string, string) (*rest.Config, error)
+	NewClientConfigFromBytes([]byte) (clientcmd.ClientConfig, error)
+	NewSPDYExecutor(*rest.Config, string, *url.URL) (remotecommand.Executor, error)
+	StreamWithContext(context.Context, remotecommand.Executor, remotecommand.StreamOptions) error
+	UntilWithSync(context.Context, cache.ListerWatcher, runtime.Object, watch2.PreconditionFunc, ...watch2.ConditionFunc) (*watch.Event, error)
+	NewFakeNeverRateLimiter() flowcontrol.RateLimiter
+	NewFakeAlwaysRateLimiter() flowcontrol.RateLimiter
+}
+
+type KubeAPIWrapper struct{}
+
+func (ku KubeAPIWrapper) NewNotFound(qualifiedResource schema.GroupResource, name string) *apierrors.StatusError {
+	return apierrors.NewNotFound(qualifiedResource, name)
+}
+
+func (ku KubeAPIWrapper) OneTermEqualSelector(k string, v string) fields.Selector {
+	return fields.OneTermEqualSelector(k, v)
+}
+
+func (ku KubeAPIWrapper) NewForConfig(c *rest.Config) (*kubernetes.Clientset, error) {
+	return kubernetes.NewForConfig(c)
+}
+
+func (ku KubeAPIWrapper) GetLogs(clientset *kubernetes.Clientset, namespace string, name string, opts *corev1.PodLogOptions) *rest.Request {
+	return clientset.CoreV1().Pods(namespace).GetLogs(name, opts)
+}
+
+func (ku KubeAPIWrapper) Get(ctx context.Context, clientset *kubernetes.Clientset, namespace string, name string, opts metav1.GetOptions) (*corev1.Pod, error) {
+	return clientset.CoreV1().Pods(namespace).Get(ctx, name, opts)
+}
+
+func (ku KubeAPIWrapper) Create(ctx context.Context, clientset *kubernetes.Clientset, namespace string, pod *corev1.Pod, opts metav1.CreateOptions) (*corev1.Pod, error) {
+	return clientset.CoreV1().Pods(namespace).Create(ctx, pod, opts)
+}
+
+func (ku KubeAPIWrapper) List(ctx context.Context, clientset *kubernetes.Clientset, namespace string, opts metav1.ListOptions) (*corev1.PodList, error) {
+	return clientset.CoreV1().Pods(namespace).List(ctx, opts)
+}
+
+func (ku KubeAPIWrapper) Watch(ctx context.Context, clientset *kubernetes.Clientset, namespace string, opts metav1.ListOptions) (watch.Interface, error) {
+	return clientset.CoreV1().Pods(namespace).Watch(ctx, opts)
+}
+
+func (ku KubeAPIWrapper) Delete(ctx context.Context, clientset *kubernetes.Clientset, namespace string, name string, opts metav1.DeleteOptions) error {
+	return clientset.CoreV1().Pods(namespace).Delete(ctx, name, opts)
+}
+
+func (ku KubeAPIWrapper) SubResource(clientset *kubernetes.Clientset, podName string, podNamespace string) *rest.Request {
+	return clientset.CoreV1().RESTClient().Post().Resource("pods").Name(podName).Namespace(podNamespace).SubResource("attach")
+}
+
+func (ku KubeAPIWrapper) InClusterConfig() (*rest.Config, error) {
+	return rest.InClusterConfig()
+}
+
+func (ku KubeAPIWrapper) NewDefaultClientConfigLoadingRules() *clientcmd.ClientConfigLoadingRules {
+	return clientcmd.NewDefaultClientConfigLoadingRules()
+}
+
+func (ku KubeAPIWrapper) BuildConfigFromFlags(masterURL string, kubeconfigPath string) (*rest.Config, error) {
+	return clientcmd.BuildConfigFromFlags(masterURL, kubeconfigPath)
+}
+
+func (ku KubeAPIWrapper) NewClientConfigFromBytes(configBytes []byte) (clientcmd.ClientConfig, error) {
+	return clientcmd.NewClientConfigFromBytes(configBytes)
+}
+
+func (ku KubeAPIWrapper) NewSPDYExecutor(config *rest.Config, method string, url *url.URL) (remotecommand.Executor, error) {
+	return remotecommand.NewSPDYExecutor(config, method, url)
+}
+
+func (ku KubeAPIWrapper) StreamWithContext(ctx context.Context, exec remotecommand.Executor, options remotecommand.StreamOptions) error {
+	return exec.StreamWithContext(ctx, options)
+}
+
+func (ku KubeAPIWrapper) UntilWithSync(ctx context.Context, lw cache.ListerWatcher, objType runtime.Object, precondition watch2.PreconditionFunc, conditions ...watch2.ConditionFunc) (*watch.Event, error) {
+	return watch2.UntilWithSync(ctx, lw, objType, precondition, conditions...)
+}
+
+func (ku KubeAPIWrapper) NewFakeNeverRateLimiter() flowcontrol.RateLimiter {
+	return flowcontrol.NewFakeNeverRateLimiter()
+}
+
+func (ku KubeAPIWrapper) NewFakeAlwaysRateLimiter() flowcontrol.RateLimiter {
+	return flowcontrol.NewFakeAlwaysRateLimiter()
+}
+
+// KubeAPIWrapperInstance is a package level var that wraps all required kubernetes API calls.
+// It is instantiated in the NewkubeWorker function and available throughout the package.
+var KubeAPIWrapperInstance KubeAPIer
+
+var KubeAPIWrapperLock *sync.RWMutex
+
 // ErrPodCompleted is returned when pod has already completed before we could attach.
 var ErrPodCompleted = fmt.Errorf("pod ran to completion")
 
@@ -78,7 +186,7 @@ func podRunningAndReady() func(event watch.Event) (bool, error) {
 	imagePullBackOffRetries := 3
 	inner := func(event watch.Event) (bool, error) {
 		if event.Type == watch.Deleted {
-			return false, apierrors.NewNotFound(schema.GroupResource{Resource: "pods"}, "")
+			return false, KubeAPIWrapperInstance.NewNotFound(schema.GroupResource{Resource: "pods"}, "")
 		}
 		if t, ok := event.Object.(*corev1.Pod); ok {
 			switch t.Status.Phase {
@@ -120,7 +228,7 @@ func podRunningAndReady() func(event watch.Event) (bool, error) {
 	return inner
 }
 
-func (kw *kubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time.Time) (io.ReadCloser, error) {
+func (kw *KubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time.Time) (io.ReadCloser, error) {
 	var logStream io.ReadCloser
 	var err error
 	podNamespace := kw.pod.Namespace
@@ -134,9 +242,7 @@ func (kw *kubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time
 		podOptions.SinceTime = &metav1.Time{Time: sinceTime}
 	}
 
-	logReq := kw.clientset.CoreV1().Pods(podNamespace).GetLogs(
-		podName, podOptions,
-	)
+	logReq := KubeAPIWrapperInstance.GetLogs(kw.clientset, podNamespace, podName, podOptions)
 	// get logstream, with retry
 	for retries := 5; retries > 0; retries-- {
 		logStream, err = logReq.Stream(kw.GetContext())
@@ -163,7 +269,7 @@ func (kw *kubeUnit) kubeLoggingConnectionHandler(timestamps bool, sinceTime time
 	return logStream, nil
 }
 
-func (kw *kubeUnit) kubeLoggingNoReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdoutErr *error) {
+func (kw *KubeUnit) kubeLoggingNoReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdoutErr *error) {
 	// Legacy method, for use on k8s < v1.23.14
 	// uses io.Copy to stream data from pod to stdout file
 	// known issues around this, as logstream can terminate due to log rotation
@@ -187,7 +293,7 @@ func (kw *kubeUnit) kubeLoggingNoReconnect(streamWait *sync.WaitGroup, stdout *S
 	}
 }
 
-func (kw *kubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdinErr *error, stdoutErr *error) {
+func (kw *KubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout *STDoutWriter, stdinErr *error, stdoutErr *error) {
 	// preferred method for k8s >= 1.23.14
 	defer streamWait.Done()
 	var sinceTime time.Time
@@ -207,7 +313,7 @@ func (kw *kubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout
 
 		// get pod, with retry
 		for retries := 5; retries > 0; retries-- {
-			kw.pod, err = kw.clientset.CoreV1().Pods(podNamespace).Get(kw.GetContext(), podName, metav1.GetOptions{})
+			kw.pod, err = KubeAPIWrapperInstance.Get(kw.GetContext(), kw.clientset, podNamespace, podName, metav1.GetOptions{})
 			if err == nil {
 				break
 			}
@@ -278,7 +384,7 @@ func (kw *kubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout
 			}
 
 			split := strings.SplitN(line, " ", 2)
-			timeStamp := parseTime(split[0])
+			timeStamp := ParseTime(split[0])
 			if !timeStamp.After(sinceTime) && !successfulWrite {
 				continue
 			}
@@ -300,8 +406,8 @@ func (kw *kubeUnit) kubeLoggingWithReconnect(streamWait *sync.WaitGroup, stdout
 	}
 }
 
-func (kw *kubeUnit) createPod(env map[string]string) error {
-	ked := kw.UnredactedStatus().ExtraData.(*kubeExtraData)
+func (kw *KubeUnit) createPod(env map[string]string) error {
+	ked := kw.UnredactedStatus().ExtraData.(*KubeExtraData)
 	command, err := shlex.Split(ked.Command)
 	if err != nil {
 		return err
@@ -383,7 +489,7 @@ func (kw *kubeUnit) createPod(env map[string]string) error {
 	}
 
 	// get pod and store to kw.pod
-	kw.pod, err = kw.clientset.CoreV1().Pods(ked.KubeNamespace).Create(kw.GetContext(), pod, metav1.CreateOptions{})
+	kw.pod, err = KubeAPIWrapperInstance.Create(kw.GetContext(), kw.clientset, ked.KubeNamespace, pod, metav1.CreateOptions{})
 	if err != nil {
 		return err
 	}
@@ -398,31 +504,33 @@ func (kw *kubeUnit) createPod(env map[string]string) error {
 		status.State = WorkStatePending
 		status.Detail = "Pod created"
 		status.StdoutSize = 0
-		status.ExtraData.(*kubeExtraData).PodName = kw.pod.Name
+		status.ExtraData.(*KubeExtraData).PodName = kw.pod.Name
 	})
 
 	// Wait for the pod to be running
-	fieldSelector := fields.OneTermEqualSelector("metadata.name", kw.pod.Name).String()
+	fieldSelector := KubeAPIWrapperInstance.OneTermEqualSelector("metadata.name", kw.pod.Name).String()
 	lw := &cache.ListWatch{
 		ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 			options.FieldSelector = fieldSelector
 
-			return kw.clientset.CoreV1().Pods(ked.KubeNamespace).List(kw.GetContext(), options)
+			return KubeAPIWrapperInstance.List(kw.GetContext(), kw.clientset, ked.KubeNamespace, options)
 		},
 		WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
 			options.FieldSelector = fieldSelector
 
-			return kw.clientset.CoreV1().Pods(ked.KubeNamespace).Watch(kw.GetContext(), options)
+			return KubeAPIWrapperInstance.Watch(kw.GetContext(), kw.clientset, ked.KubeNamespace, options)
 		},
 	}
 
 	ctxPodReady := kw.GetContext()
 	if kw.podPendingTimeout != time.Duration(0) {
-		ctxPodReady, _ = context.WithTimeout(kw.GetContext(), kw.podPendingTimeout)
+		var ctxPodCancel context.CancelFunc
+		ctxPodReady, ctxPodCancel = context.WithTimeout(kw.GetContext(), kw.podPendingTimeout)
+		defer ctxPodCancel()
 	}
 
 	time.Sleep(2 * time.Second)
-	ev, err := watch2.UntilWithSync(ctxPodReady, lw, &corev1.Pod{}, nil, podRunningAndReady())
+	ev, err := KubeAPIWrapperInstance.UntilWithSync(ctxPodReady, lw, &corev1.Pod{}, nil, podRunningAndReady())
 	if ev == nil || ev.Object == nil {
 		return fmt.Errorf("did not return an event while watching pod for work unit %s", kw.ID())
 	}
@@ -487,11 +595,11 @@ func (kw *kubeUnit) createPod(env map[string]string) error {
 	return nil
 }
 
-func (kw *kubeUnit) runWorkUsingLogger() {
+func (kw *KubeUnit) runWorkUsingLogger() {
 	skipStdin := true
 
 	status := kw.Status()
-	ked := status.ExtraData.(*kubeExtraData)
+	ked := status.ExtraData.(*KubeExtraData)
 
 	podName := ked.PodName
 	podNamespace := ked.KubeNamespace
@@ -538,7 +646,7 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 			default:
 			}
 
-			kw.pod, err = kw.clientset.CoreV1().Pods(podNamespace).Get(kw.GetContext(), podName, metav1.GetOptions{})
+			kw.pod, err = KubeAPIWrapperInstance.Get(kw.GetContext(), kw.clientset, podNamespace, podName, metav1.GetOptions{})
 			if err == nil {
 				break
 			}
@@ -563,11 +671,7 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 	// Attach stdin stream to the pod
 	var exec remotecommand.Executor
 	if !skipStdin {
-		req := kw.clientset.CoreV1().RESTClient().Post().
-			Resource("pods").
-			Name(podName).
-			Namespace(podNamespace).
-			SubResource("attach")
+		req := KubeAPIWrapperInstance.SubResource(kw.clientset, podName, podNamespace)
 
 		req.VersionedParams(
 			&corev1.PodExecOptions{
@@ -579,9 +683,8 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 			},
 			scheme.ParameterCodec,
 		)
-
 		var err error
-		exec, err = remotecommand.NewSPDYExecutor(kw.config, "POST", req.URL())
+		exec, err = KubeAPIWrapperInstance.NewSPDYExecutor(kw.config, "POST", req.URL())
 		if err != nil {
 			errMsg := fmt.Sprintf("Error creating SPDY executor: %s", err)
 			kw.UpdateBasicStatus(WorkStateFailed, errMsg, 0)
@@ -675,7 +778,7 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 
 			var err error
 			for retries := 5; retries > 0; retries-- {
-				err = exec.StreamWithContext(kw.GetContext(), remotecommand.StreamOptions{
+				err = KubeAPIWrapperInstance.StreamWithContext(kw.GetContext(), exec, remotecommand.StreamOptions{
 					Stdin: stdin,
 					Tty:   false,
 				})
@@ -721,7 +824,7 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 		}()
 	}
 
-	stdoutWithReconnect := shouldUseReconnect()
+	stdoutWithReconnect := ShouldUseReconnect(kw)
 	if stdoutWithReconnect && stdoutErr == nil {
 		kw.GetWorkceptor().nc.GetLogger().Debug("streaming stdout with reconnect support")
 		go kw.kubeLoggingWithReconnect(&streamWait, stdout, &stdinErr, &stdoutErr)
@@ -757,11 +860,54 @@ func (kw *kubeUnit) runWorkUsingLogger() {
 	}
 }
 
-func shouldUseReconnect() bool {
+func IsCompatibleK8S(kw *KubeUnit, versionStr string) bool {
+	semver, err := version.ParseSemantic(versionStr)
+	if err != nil {
+		kw.GetWorkceptor().nc.GetLogger().Warning("could parse Kubernetes server version %s, will not use reconnect support", versionStr)
+
+		return false
+	}
+
+	// ignore pre-release in version comparison
+	semver = semver.WithPreRelease("")
+
+	// The patch was backported to minor version 23, 24 and 25
+	// We check z stream based on the minor version
+	// if minor versions < 23, set to high value (e.g. v1.22.9999)
+	// if minor versions == 23, compare with v1.23.14
+	// if minor version == 24, compare with v1.24.8
+	// if minor version == 25, compare with v1.25.4
+	// if minor versions > 23, compare with low value (e.g. v1.26.0)
+	var compatibleVer string
+	switch {
+	case semver.Minor() == 23:
+		compatibleVer = "v1.23.14"
+	case semver.Minor() == 24:
+		compatibleVer = "v1.24.8"
+	case semver.Minor() == 25:
+		compatibleVer = "v1.25.4"
+	case semver.Minor() > 25:
+		compatibleVer = fmt.Sprintf("%d.%d.0", semver.Major(), semver.Minor())
+	default:
+		compatibleVer = fmt.Sprintf("%d.%d.9999", semver.Major(), semver.Minor())
+	}
+
+	if semver.AtLeast(version.MustParseSemantic(compatibleVer)) {
+		kw.GetWorkceptor().nc.GetLogger().Debug("Kubernetes version %s is at least %s, using reconnect support", semver, compatibleVer)
+
+		return true
+	}
+
+	kw.GetWorkceptor().nc.GetLogger().Debug("Kubernetes version %s not at least %s, not using reconnect support", semver, compatibleVer)
+
+	return false
+}
+
+func ShouldUseReconnect(kw *KubeUnit) bool {
 	// Support for streaming from pod with timestamps using reconnect method is in all current versions
 	// Can override the detection by setting the RECEPTOR_KUBE_SUPPORT_RECONNECT
-	// accepted values: "enabled", "disabled", "auto" with "disabled" being the default
-	// all invalid value will assume to be "disabled"
+	// accepted values: "enabled", "disabled", "auto".  The default is "enabled"
+	// all invalid values will assume to be "disabled"
 
 	env, ok := os.LookupEnv("RECEPTOR_KUBE_SUPPORT_RECONNECT")
 	if ok {
@@ -771,16 +917,23 @@ func shouldUseReconnect() bool {
 		case "disabled":
 			return false
 		case "auto":
-			// continue
+			return true
 		default:
 			return false
 		}
 	}
 
-	return false
+	serverVerInfo, err := kw.clientset.ServerVersion()
+	if err != nil {
+		kw.GetWorkceptor().nc.GetLogger().Warning("could not detect Kubernetes server version, will not use reconnect support")
+
+		return false
+	}
+
+	return IsCompatibleK8S(kw, serverVerInfo.String())
 }
 
-func parseTime(s string) *time.Time {
+func ParseTime(s string) *time.Time {
 	t, err := time.Parse(time.RFC3339, s)
 	if err == nil {
 		return &t
@@ -820,7 +973,7 @@ func getDefaultInterface() (string, error) {
 	return "", fmt.Errorf("could not determine local address")
 }
 
-func (kw *kubeUnit) runWorkUsingTCP() {
+func (kw *KubeUnit) runWorkUsingTCP() {
 	// Create local cancellable context
 	ctx, cancel := kw.GetContext(), kw.GetCancel()
 	defer cancel()
@@ -971,12 +1124,12 @@ func (kw *kubeUnit) runWorkUsingTCP() {
 	}
 }
 
-func (kw *kubeUnit) connectUsingKubeconfig() error {
+func (kw *KubeUnit) connectUsingKubeconfig() error {
 	var err error
-	ked := kw.UnredactedStatus().ExtraData.(*kubeExtraData)
+	ked := kw.UnredactedStatus().ExtraData.(*KubeExtraData)
 	if ked.KubeConfig == "" {
-		clr := clientcmd.NewDefaultClientConfigLoadingRules()
-		kw.config, err = clientcmd.BuildConfigFromFlags("", clr.GetDefaultFilename())
+		clr := KubeAPIWrapperInstance.NewDefaultClientConfigLoadingRules()
+		kw.config, err = KubeAPIWrapperInstance.BuildConfigFromFlags("", clr.GetDefaultFilename())
 		if ked.KubeNamespace == "" {
 			c, err := clr.Load()
 			if err != nil {
@@ -985,14 +1138,14 @@ func (kw *kubeUnit) connectUsingKubeconfig() error {
 			curContext, ok := c.Contexts[c.CurrentContext]
 			if ok && curContext != nil {
 				kw.UpdateFullStatus(func(sfd *StatusFileData) {
-					sfd.ExtraData.(*kubeExtraData).KubeNamespace = curContext.Namespace
+					sfd.ExtraData.(*KubeExtraData).KubeNamespace = curContext.Namespace
 				})
 			} else {
 				return fmt.Errorf("could not determine namespace")
 			}
 		}
 	} else {
-		cfg, err := clientcmd.NewClientConfigFromBytes([]byte(ked.KubeConfig))
+		cfg, err := KubeAPIWrapperInstance.NewClientConfigFromBytes([]byte(ked.KubeConfig))
 		if err != nil {
 			return err
 		}
@@ -1002,7 +1155,7 @@ func (kw *kubeUnit) connectUsingKubeconfig() error {
 				return err
 			}
 			kw.UpdateFullStatus(func(sfd *StatusFileData) {
-				sfd.ExtraData.(*kubeExtraData).KubeNamespace = namespace
+				sfd.ExtraData.(*KubeExtraData).KubeNamespace = namespace
 			})
 		}
 		kw.config, err = cfg.ClientConfig()
@@ -1017,9 +1170,9 @@ func (kw *kubeUnit) connectUsingKubeconfig() error {
 	return nil
 }
 
-func (kw *kubeUnit) connectUsingIncluster() error {
+func (kw *KubeUnit) connectUsingIncluster() error {
 	var err error
-	kw.config, err = rest.InClusterConfig()
+	kw.config, err = KubeAPIWrapperInstance.InClusterConfig()
 	if err != nil {
 		return err
 	}
@@ -1027,7 +1180,7 @@ func (kw *kubeUnit) connectUsingIncluster() error {
 	return nil
 }
 
-func (kw *kubeUnit) connectToKube() error {
+func (kw *KubeUnit) connectToKube() error {
 	var err error
 	switch {
 	case kw.authMethod == "kubeconfig" || kw.authMethod == "runtime":
@@ -1082,16 +1235,16 @@ func (kw *kubeUnit) connectToKube() error {
 	if ok {
 		switch envRateLimiter {
 		case "never":
-			kw.config.RateLimiter = flowcontrol.NewFakeNeverRateLimiter()
+			kw.config.RateLimiter = KubeAPIWrapperInstance.NewFakeNeverRateLimiter()
 		case "always":
-			kw.config.RateLimiter = flowcontrol.NewFakeAlwaysRateLimiter()
+			kw.config.RateLimiter = KubeAPIWrapperInstance.NewFakeAlwaysRateLimiter()
 		default:
 		}
 		kw.GetWorkceptor().nc.GetLogger().Debug("RateLimiter: %s", envRateLimiter)
 	}
 
 	kw.GetWorkceptor().nc.GetLogger().Debug("QPS: %f, Burst: %d", kw.config.QPS, kw.config.Burst)
-	kw.clientset, err = kubernetes.NewForConfig(kw.config)
+	kw.clientset, err = KubeAPIWrapperInstance.NewForConfig(kw.config)
 	if err != nil {
 		return err
 	}
@@ -1113,8 +1266,8 @@ func readFileToString(filename string) (string, error) {
 }
 
 // SetFromParams sets the in-memory state from parameters.
-func (kw *kubeUnit) SetFromParams(params map[string]string) error {
-	ked := kw.GetStatusCopy().ExtraData.(*kubeExtraData)
+func (kw *KubeUnit) SetFromParams(params map[string]string) error {
+	ked := kw.GetStatusCopy().ExtraData.(*KubeExtraData)
 	type value struct {
 		name       string
 		permission bool
@@ -1201,9 +1354,9 @@ func (kw *kubeUnit) SetFromParams(params map[string]string) error {
 }
 
 // Status returns a copy of the status currently loaded in memory.
-func (kw *kubeUnit) Status() *StatusFileData {
+func (kw *KubeUnit) Status() *StatusFileData {
 	status := kw.UnredactedStatus()
-	ed, ok := status.ExtraData.(*kubeExtraData)
+	ed, ok := status.ExtraData.(*KubeExtraData)
 	if ok {
 		ed.KubeConfig = ""
 		ed.KubePod = ""
@@ -1213,11 +1366,11 @@ func (kw *kubeUnit) Status() *StatusFileData {
 }
 
 // Status returns a copy of the status currently loaded in memory.
-func (kw *kubeUnit) UnredactedStatus() *StatusFileData {
+func (kw *KubeUnit) UnredactedStatus() *StatusFileData {
 	kw.GetStatusLock().RLock()
 	defer kw.GetStatusLock().RUnlock()
 	status := kw.GetStatusWithoutExtraData()
-	ked, ok := kw.GetStatusCopy().ExtraData.(*kubeExtraData)
+	ked, ok := kw.GetStatusCopy().ExtraData.(*KubeExtraData)
 	if ok {
 		kedCopy := *ked
 		status.ExtraData = &kedCopy
@@ -1227,7 +1380,7 @@ func (kw *kubeUnit) UnredactedStatus() *StatusFileData {
 }
 
 // startOrRestart is a shared implementation of Start() and Restart().
-func (kw *kubeUnit) startOrRestart() error {
+func (kw *KubeUnit) startOrRestart() error {
 	// Connect to the Kubernetes API
 	if err := kw.connectToKube(); err != nil {
 		return err
@@ -1244,9 +1397,9 @@ func (kw *kubeUnit) startOrRestart() error {
 }
 
 // Restart resumes monitoring a job after a Receptor restart.
-func (kw *kubeUnit) Restart() error {
+func (kw *KubeUnit) Restart() error {
 	status := kw.Status()
-	ked := status.ExtraData.(*kubeExtraData)
+	ked := status.ExtraData.(*KubeExtraData)
 	if IsComplete(status.State) {
 		return nil
 	}
@@ -1260,7 +1413,7 @@ func (kw *kubeUnit) Restart() error {
 		if err != nil {
 			kw.GetWorkceptor().nc.GetLogger().Warning("Pod %s could not be deleted: %s", ked.PodName, err.Error())
 		} else {
-			err := kw.clientset.CoreV1().Pods(ked.KubeNamespace).Delete(context.Background(), ked.PodName, metav1.DeleteOptions{})
+			err := KubeAPIWrapperInstance.Delete(context.Background(), kw.clientset, ked.KubeNamespace, ked.PodName, metav1.DeleteOptions{})
 			if err != nil {
 				kw.GetWorkceptor().nc.GetLogger().Warning("Pod %s could not be deleted: %s", ked.PodName, err.Error())
 			}
@@ -1274,18 +1427,18 @@ func (kw *kubeUnit) Restart() error {
 }
 
 // Start launches a job with given parameters.
-func (kw *kubeUnit) Start() error {
+func (kw *KubeUnit) Start() error {
 	kw.UpdateBasicStatus(WorkStatePending, "Connecting to Kubernetes", 0)
 
 	return kw.startOrRestart()
 }
 
 // Cancel releases resources associated with a job, including cancelling it if running.
-func (kw *kubeUnit) Cancel() error {
+func (kw *KubeUnit) Cancel() error {
 	kw.CancelContext()
 	kw.UpdateBasicStatus(WorkStateCanceled, "Canceled", -1)
 	if kw.pod != nil {
-		err := kw.clientset.CoreV1().Pods(kw.pod.Namespace).Delete(context.Background(), kw.pod.Name, metav1.DeleteOptions{})
+		err := KubeAPIWrapperInstance.Delete(context.Background(), kw.clientset, kw.pod.Namespace, kw.pod.Name, metav1.DeleteOptions{})
 		if err != nil {
 			kw.GetWorkceptor().nc.GetLogger().Error("Error deleting pod %s: %s", kw.pod.Name, err)
 		}
@@ -1298,7 +1451,7 @@ func (kw *kubeUnit) Cancel() error {
 }
 
 // Release releases resources associated with a job.  Implies Cancel.
-func (kw *kubeUnit) Release(force bool) error {
+func (kw *KubeUnit) Release(force bool) error {
 	err := kw.Cancel()
 	if err != nil && !force {
 		return err
@@ -1332,10 +1485,14 @@ type KubeWorkerCfg struct {
 
 // NewWorker is a factory to produce worker instances.
 func (cfg KubeWorkerCfg) NewWorker(bwu BaseWorkUnitForWorkUnit, w *Workceptor, unitID string, workType string) WorkUnit {
+	return cfg.NewkubeWorker(bwu, w, unitID, workType, nil)
+}
+
+func (cfg KubeWorkerCfg) NewkubeWorker(bwu BaseWorkUnitForWorkUnit, w *Workceptor, unitID string, workType string, kawi KubeAPIer) WorkUnit {
 	if bwu == nil {
 		bwu = &BaseWorkUnit{
 			status: StatusFileData{
-				ExtraData: &kubeExtraData{
+				ExtraData: &KubeExtraData{
 					Image:         cfg.Image,
 					Command:       cfg.Command,
 					KubeNamespace: cfg.Namespace,
@@ -1346,7 +1503,15 @@ func (cfg KubeWorkerCfg) NewWorker(bwu BaseWorkUnitForWorkUnit, w *Workceptor, u
 		}
 	}
 
-	ku := &kubeUnit{
+	KubeAPIWrapperLock = &sync.RWMutex{}
+	KubeAPIWrapperLock.Lock()
+	KubeAPIWrapperInstance = KubeAPIWrapper{}
+	if kawi != nil {
+		KubeAPIWrapperInstance = kawi
+	}
+	KubeAPIWrapperLock.Unlock()
+
+	ku := &KubeUnit{
 		BaseWorkUnitForWorkUnit: bwu,
 		authMethod:              strings.ToLower(cfg.AuthMethod),
 		streamMethod:            strings.ToLower(cfg.StreamMethod),
