@@ -33,8 +33,10 @@ func TestWorkSubmitWithTLSClient(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.DataDir)
 			}
-			ctx, _ := context.WithTimeout(context.Background(), 60*time.Second)
-			err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 60*time.Second)
+			defer cancel1()
+
+			err = controllers["node1"].AssertWorkSucceeded(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.DataDir)
 			}
@@ -66,29 +68,31 @@ func TestWorkSubmitWithIncorrectTLSClient(t *testing.T) {
 				t.Fatal(err)
 			}
 
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkFailed(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
 
+			err = controllers["node1"].AssertWorkFailed(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
 
 			_, err = controllers["node1"].WorkRelease(unitID)
-
 			if err != nil {
 				t.Fatal(err)
 			}
 
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkReleased(ctx, unitID)
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
 
+			err = controllers["node1"].AssertWorkReleased(ctx2, unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
 
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = assertFilesReleased(ctx, nodes["node1"].GetDataDir(), "node1", unitID)
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
 
+			err = assertFilesReleased(ctx3, nodes["node1"].GetDataDir(), "node1", unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
@@ -119,8 +123,10 @@ func TestStartRemoteWorkWithTTL(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			ctx, _ := context.WithTimeout(context.Background(), 10*time.Second)
-			err = controllers["node1"].AssertWorkTimedOut(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 10*time.Second)
+			defer cancel1()
+
+			err = controllers["node1"].AssertWorkTimedOut(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
@@ -128,13 +134,17 @@ func TestStartRemoteWorkWithTTL(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 5*time.Second)
-			err = controllers["node1"].AssertWorkReleased(ctx, unitID)
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 5*time.Second)
+			defer cancel2()
+
+			err = controllers["node1"].AssertWorkReleased(ctx2, unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 5*time.Second)
-			err = assertFilesReleased(ctx, nodes["node1"].GetDataDir(), "node1", unitID)
+
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 5*time.Second)
+			defer cancel3()
+			err = assertFilesReleased(ctx3, nodes["node1"].GetDataDir(), "node1", unitID)
 			if err != nil {
 				t.Fatal(err)
 			}
@@ -162,8 +172,10 @@ func TestCancelThenReleaseRemoteWork(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkRunning(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
+
+			err = controllers["node1"].AssertWorkRunning(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -171,8 +183,11 @@ func TestCancelThenReleaseRemoteWork(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkCancelled(ctx, unitID)
+
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
+
+			err = controllers["node1"].AssertWorkCancelled(ctx2, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -189,8 +204,10 @@ func TestCancelThenReleaseRemoteWork(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = m.WaitForReady(ctx)
+
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
+			err = m.WaitForReady(ctx3)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -206,18 +223,26 @@ func TestCancelThenReleaseRemoteWork(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkReleased(ctx, unitID)
+			ctx4, cancel4 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel4()
+
+			err = controllers["node1"].AssertWorkReleased(ctx4, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = assertFilesReleased(ctx, nodes["node1"].GetDataDir(), "node1", unitID)
+
+			ctx5, cancel5 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel5()
+
+			err = assertFilesReleased(ctx5, nodes["node1"].GetDataDir(), "node1", unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = assertFilesReleased(ctx, nodes["node3"].GetDataDir(), "node3", remoteUnitID)
+
+			ctx6, cancel6 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel6()
+
+			err = assertFilesReleased(ctx6, nodes["node3"].GetDataDir(), "node3", remoteUnitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -242,30 +267,32 @@ func TestWorkSubmitWhileRemoteNodeIsDown(t *testing.T) {
 				t.Fatal(err, m.GetDataDir())
 			}
 
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkPending(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
 
+			err = controllers["node1"].AssertWorkPending(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
 
 			err = nodes["node3"].Start()
-
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
 
 			// Wait for node3 to join the mesh again
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = m.WaitForReady(ctx)
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
 
+			err = m.WaitForReady(ctx2)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
 
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
 
+			err = controllers["node1"].AssertWorkSucceeded(ctx3, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -293,13 +320,17 @@ func TestWorkStreamingResumesWhenRelayNodeRestarts(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkRunning(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
+
+			err = controllers["node1"].AssertWorkRunning(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = assertStdoutFizeSize(ctx, nodes["node1"].GetDataDir(), "node1", unitID, 1)
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
+
+			err = assertStdoutFizeSize(ctx2, nodes["node1"].GetDataDir(), "node1", unitID, 1)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -310,19 +341,25 @@ func TestWorkStreamingResumesWhenRelayNodeRestarts(t *testing.T) {
 			nodes["node2"].Shutdown()
 			nodes["node2"].Start()
 			// Wait for node2 to join the mesh again
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = m.WaitForReady(ctx)
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
+
+			err = m.WaitForReady(ctx3)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
 
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
+			ctx4, cancel4 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel4()
+
+			err = controllers["node1"].AssertWorkSucceeded(ctx4, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = assertStdoutFizeSize(ctx, nodes["node1"].GetDataDir(), "node1", unitID, 10)
+			ctx5, cancel5 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel5()
+
+			err = assertStdoutFizeSize(ctx5, nodes["node1"].GetDataDir(), "node1", unitID, 10)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -349,8 +386,9 @@ func TestResultsOnRestartedNode(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkRunning(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
+			err = controllers["node1"].AssertWorkRunning(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -361,14 +399,18 @@ func TestResultsOnRestartedNode(t *testing.T) {
 				t.Fatal(err, m.GetDataDir())
 			}
 			// Wait for node3 to join the mesh again
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = m.WaitForReady(ctx)
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
+
+			err = m.WaitForReady(ctx2)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
 
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
+
+			err = controllers["node1"].AssertWorkSucceeded(ctx3, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -399,8 +441,10 @@ func TestWorkSubmitAndReleaseToNonexistentNode(t *testing.T) {
 			}
 
 			// wait for 10 seconds, and check if the work is in pending state
-			ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkPending(ctx, unitID)
+			ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel1()
+
+			err = controllers["node1"].AssertWorkPending(ctx1, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -410,8 +454,11 @@ func TestWorkSubmitAndReleaseToNonexistentNode(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = m.WaitForReady(ctx)
+
+			ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel2()
+
+			err = m.WaitForReady(ctx2)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -429,8 +476,11 @@ func TestWorkSubmitAndReleaseToNonexistentNode(t *testing.T) {
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
-			ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-			err = controllers["node1"].AssertWorkReleased(ctx, unitID)
+
+			ctx3, cancel3 := context.WithTimeout(context.Background(), 120*time.Second)
+			defer cancel3()
+
+			err = controllers["node1"].AssertWorkReleased(ctx3, unitID)
 			if err != nil {
 				t.Fatal(err, m.GetDataDir())
 			}
@@ -454,9 +504,9 @@ func TestRuntimeParams(t *testing.T) {
 		t.Fatal(err, m.GetDataDir())
 	}
 
-	ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-	err = m.WaitForReady(ctx)
-
+	ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel1()
+	err = m.WaitForReady(ctx1)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -465,7 +515,6 @@ func TestRuntimeParams(t *testing.T) {
 	controllers := make(map[string]*ReceptorControl)
 	controllers["node1"] = NewReceptorControl()
 	err = controllers["node1"].Connect(nodes["node1"].GetControlSocket())
-
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -476,14 +525,12 @@ func TestRuntimeParams(t *testing.T) {
 		t.Fatal(err, m.GetDataDir())
 	}
 
-	err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
-
+	err = controllers["node1"].AssertWorkSucceeded(ctx1, unitID)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
 
 	err = controllers["node1"].AssertWorkResults(unitID, []byte("it worked!"))
-
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -506,8 +553,10 @@ func TestKubeRuntimeParams(t *testing.T) {
 
 	m.Start(t.Name())
 
-	ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-	err := m.WaitForReady(ctx)
+	ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel1()
+
+	err := m.WaitForReady(ctx1)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -528,7 +577,6 @@ func TestKubeRuntimeParams(t *testing.T) {
 		"secret_kube_pod": "%s",
 		"secret_kube_config": "%s"
     }`))
-
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -556,14 +604,12 @@ func TestKubeRuntimeParams(t *testing.T) {
 		t.Fatal(err, m.GetDataDir())
 	}
 
-	err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
-
+	err = controllers["node1"].AssertWorkSucceeded(ctx1, unitID)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
 
 	err = controllers["node1"].AssertWorkResults(unitID, []byte("1\n2\n3\n4\n5\n"))
-
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -585,8 +631,10 @@ func TestRuntimeParamsNotAllowed(t *testing.T) {
 		t.Fatal(err, m.GetDataDir())
 	}
 
-	ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-	err = m.WaitForReady(ctx)
+	ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel1()
+
+	err = m.WaitForReady(ctx1)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -627,8 +675,10 @@ func TestKubeContainerFailure(t *testing.T) {
 
 	m.Start(t.Name())
 
-	ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-	err := m.WaitForReady(ctx)
+	ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel1()
+
+	err := m.WaitForReady(ctx1)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -644,8 +694,10 @@ func TestKubeContainerFailure(t *testing.T) {
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
-	ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-	err = controllers["node1"].AssertWorkFailed(ctx, unitID)
+	ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel2()
+
+	err = controllers["node1"].AssertWorkFailed(ctx2, unitID)
 	if err != nil {
 		t.Fatal("Expected work to fail but it succeeded")
 	}
@@ -715,13 +767,14 @@ func TestSignedWorkVerification(t *testing.T) {
 	}
 
 	err = m.Start(t.Name())
-
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	ctx, _ := context.WithTimeout(context.Background(), 120*time.Second)
-	err = m.WaitForReady(ctx)
+	ctx1, cancel1 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel1()
+
+	err = m.WaitForReady(ctx1)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
@@ -738,13 +791,15 @@ func TestSignedWorkVerification(t *testing.T) {
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
-	ctx, _ = context.WithTimeout(context.Background(), 120*time.Second)
-	err = controllers["node1"].AssertWorkSucceeded(ctx, unitID)
+	ctx2, cancel2 := context.WithTimeout(context.Background(), 120*time.Second)
+	defer cancel2()
+
+	err = controllers["node1"].AssertWorkSucceeded(ctx2, unitID)
 	if err != nil {
 		t.Fatal(err, m.GetDataDir())
 	}
 
-	// node2 has the wrong public key to verify work signatures, so the work submission should fail
+	// node3 has the wrong public key to verify work signatures, so the work submission should fail
 	job = `{"command":"work","subcommand":"submit","worktype":"echo","node":"node3", "signwork":"true"}`
 	_, err = controllers["node1"].WorkSubmitJSON(job)
 	if err == nil {
